#!/usr/bin/env python
'''
navi.py
Peter Mawhorter
2008-1-8

This is the navi server for the heylisten program. It accepts connections on
port 51422 and receives data messages that contain the peak frequency values
for a sound stream. The server then performs recognition on the peak-values
stream and performs specified actions for recognized note sequences.
'''

import socket, Queue, threading, time

import recognize

port = 51422

batchsize = 3 # Number of peaks per message. Must match heylisten's main.c.

def listen(port, queue):
  '''
  This function listens on the given port (it only accepts one connection at a
  time) and handles messages describing peak audio input frequencies. It
  converts the messages into integers and puts them into a queue for
  recognition.
  '''
  sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
  sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
  sock.bind(("127.0.0.1", port))
  sock.listen(1)
  while True:
    conn, addr = sock.accept()
    msg = ''
    done = False
    while not done:
      oldlen = len(msg)
      msg += conn.recv(128)
      if len(msg) == oldlen:
        time.sleep(0.05)
      while msg:
        type = msg[0]
        if type == 'P': # Peaks
          if msg[batchsize*2 + 1:]:
            msg = msg[1:]
            #print "batch..."
            for i in range(batchsize):
              #print "putting:", (ord(msg[i*2]) << 8) + ord(msg[i*2+1])
              queue.put((ord(msg[i*2]) << 8) + ord(msg[i*2+1]))
            msg = msg[batchsize*2:]
          else: # Need more data to continue
            break
        elif type == 'D': # Drop
          done = True
          msg = msg[1:]
          break
    conn.close()
  sock.close()

if __name__ == "__main__":
  queue = Queue.Queue(0)
  thread = threading.Thread(group=None, target=recognize.attend,
                            name="Recognizer", args=(queue,))
  thread.daemon = True
  thread.start()
  try:
    listen(port, queue)
  except:
    pass
  queue.put('Q')
  thread.join()
